/*
** Module   : MenuChuiImpl.java
** Abstract : CHUI client implementation of Menu.
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------------Description-----------------------------
** 001 VIG 20150116 Created initial version.
** 002 VIG 20150225 Added event processing and drawing methods.
*/
package com.goldencode.p2j.ui.client.chui;

import java.util.List;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.chui.ThinClient;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.chui.driver.ChuiOutputManager;
import com.goldencode.p2j.ui.client.event.*;
import com.goldencode.p2j.ui.client.widget.*;

/**
 * <code>Menu</code> CHUI client implementation class. 
 */
public class MenuItemChuiImpl
extends MenuItem<ChuiOutputManager>
{
   /**
    * Special constructor used to restore component using given configuration.
    * 
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Configuration data.
    */
   public MenuItemChuiImpl(WidgetId id, WidgetConfig cfg)
   {
      super(id, (MenuItemConfig) cfg);
   }

   /**
    * Draw the peer container. The methods calls the draw() method for
    * ScrollPane instance for the peer container. In addition the required
    * clipping rectangle is calculated for ChildFrame containers because they
    * must be rendered inside the parent container.
    * <p>
    * Drawing for the redirected terminal differs. Labels/headers are drawn
    * only once per page, instead of the usual once per menu.
    */
   @Override
   public void draw()
   {
      Color dColor = config().dcolor == null ? Color.NORMAL : config().dcolor;
      
      Point p = calcPosition();
      
      drawTitle(p, dColor);
      
      EventManager.postEvent(new PaintEvent(parent(), parent().bounds()));
   }

   /**
    * Get widget dimension.
    * 
    * @return  widget dimension.
    */
   @Override
   public Dimension dimension()
   {
      return new Dimension(width(), 1);
   }

   /**
    * Get widget height.
    * 
    * @return  widget height.
    */
   @Override
   public double height()
   {
      return 1;
   }

   /**
    * Get widget minimum size.
    * 
    * @return  widget minimum size.
    */
   @Override
   public Dimension minimumSize()
   {
      return dimension();
   }

   /**
    * Get widget width.
    * 
    * @return  widget width.
    */
   @Override
   public double width()
   {
      String title = config().title;
      int w = title.length();
      return title.contains("&") ? w - 1 : w;
   }

   /**
    * Process key event.
    * 
    * @param   event
    *          Key event to process.
    */
   @Override
   public void processKeyEvent(KeyInput event)
   {
      if (event.isConsumed())
      {
         return;
      }

      int key = event.keyCode();
      int actionCode = event.actionCode();
      
      if (key == Key.VK_ENTER ||
         (isMenubar() && actionCode == Keyboard.KA_CURSOR_DOWN) ||       
         (isMenubar() && actionCode == Keyboard.KA_CURSOR_UP))
      {
         EventManager.postEvent(new ItemEvent(this));
         return;
      }

      if ((!isMenubar() && actionCode == Keyboard.KA_CURSOR_DOWN) || 
          (isMenubar() && actionCode == Keyboard.KA_CURSOR_RIGHT))
      {
         ThinClient.nextFocus(this);
         EventManager.postEvent(new PaintEvent(this, this.bounds()));
         return;
      }
      
      if ((!isMenubar() && actionCode == Keyboard.KA_CURSOR_UP) || 
          (isMenubar() && actionCode == Keyboard.KA_CURSOR_LEFT))
      {
         ThinClient.previousFocus(this);
         return;
      }

      // mnemonic
      if (event.keyChar() != KeyInput.CHAR_UNDEFINED || event.isRealKey())
      {
         parent().processKeyEvent(event);
      }
      else
      {
         super.processKeyEvent(event);
      }
      
   }

   /**
    * Check if widget needs in cursor to be on when having the input focus.  By default this
    * method always returns <code>true</code> so it needs to be overwritten if other return value
    * is required for particular widget implementation class.
    * 
    * @return  <code>true</code> as the default value.
    */
   @Override
   public boolean forceCursorOnFocus()
   {
      return false;
   }

   /**
    * Calculates item position.
    * 
    * @return  new Point of this item.
    */
   private Point calcPosition()
   {
      AbstractContainer<ChuiOutputManager> parent = (AbstractContainer<ChuiOutputManager>) parent();      
      
      List<Widget<ChuiOutputManager>> widgets = parent.getWidgetsList();

      int idx = widgets.indexOf(this);
      
      if (idx == -1) 
      {
         throw new IllegalStateException("MENU-ITEM is not contained in its parents widgets list");
      }
      
      Point p = parent.location();
      
      double x = p.x;
      double y = p.y;
      
      if (parent instanceof Menu && !((Menu<?>)parent).config().popupOnly)
      {
         for(Widget<?> w : widgets)
         {
            if (w.equals(this))
               break;
            x += 1 + w.width();
         }
      }
      else
      {
         x += 2;
         y += 2 + idx;
      }
      
      p = new Point(x, y);
      return p;
   }

   /**
    * Draws menu-item title, drawing mnemonic character as underlined.
    * 
    * @param   p
    *          Title point.  
    * @param   dColor
    *          Color of the title.
    */
   private void drawTitle(Point p, Color dColor)
   {
      if (hasFocus())
      {
         dColor = dColor.withReverse();
      }
      
      String title = this.config().title;
      int ampIdx = title.indexOf('&');
      if (ampIdx != -1)
      {
         Point tP = new Point(p);
         String leftPart = title.substring(0, ampIdx);
         screen().at(tP).append(leftPart, dColor);
         
         tP.x += ampIdx;
         Color mnemonicColor = dColor.withUnderline().withoutReverse();
         screen().at(tP).append(title.substring(ampIdx + 1, ampIdx + 2), mnemonicColor);
         
         tP.x += 1;
         screen().at(tP).append(title.substring(ampIdx+2), dColor);
      }
      else
      {
         screen().at(p).append(title, dColor);
      }
   }

   /**
    * Check if this sub-menu is right inside MENUBAR (first level)
    * 
    * @return  <code>true</code> if yes.
    */
   private boolean isMenubar()
   {
      Container<ChuiOutputManager> parent = parent();
      return parent instanceof Menu && !((Menu<?>)parent).config().popupOnly;
   }
}
